/******************************************************************************
*
* project name: TI-MineSweeper
* filename:     cursor.c
* initial date: 04/07/2000
* authors:      thomas.nussbaumer@gmx.net (coding)
*               marcos.lopez@gmx.net (game play/graphics/BETA testing)
* description:  cursor handling routines and cursor data
*
* $Id: cursor.c,v 1.1 2002/09/27 10:10:29 tnussb Exp $
*
*******************************************************************************/

extern unsigned char* plane1;
extern unsigned char* plane2;
extern unsigned short tiles_offset_x;
extern unsigned short tiles_offset_y;
extern SCR_RECT       fullscreen;

unsigned char pointer_plane1[] = {0x80,0xc0,0xe0,0xb0,0xb8,0x9c,0x9e,0xb8,0xdc,0x9c,0x0e,0x0e};
unsigned char pointer_plane2[] = {0x80,0xc0,0xa0,0x90,0x88,0x84,0x8e,0xa8,0xd4,0x94,0x0a,0x0e};
unsigned char mask_plane1[]    = {~0x00,~0x00,~0x40,~0x60,~0x70,~0x78,~0x70,~0x50,~0x08,~0x08,~0x04,~0x00};
unsigned char mask_plane2[]    = {~0x00,~0x00,~0x40,~0x60,~0x70,~0x78,~0x70,~0x50,~0x08,~0x08,~0x04,~0x00};


SCR_RECT old_rect = {{ 255,255,255,255 }};


extern unsigned char* cursor_buf1;
extern unsigned char* cursor_buf2;


/*===========================================================================*/
/* sets x0 to 255 to indicate that buffer contains no useful data            */
/*===========================================================================*/
void InitCursorBuffer() {
    old_rect.xy.x0 = 255;
}


/*===========================================================================*/
/* restores background of old cursor position                                */
/*===========================================================================*/
void RestoreOldCursorPos() {
    //-----------------------------------------------------------------
    // restore only if buffer contains data (x0==255 indicates no data)
    //-----------------------------------------------------------------
    if (old_rect.xy.x0 != 255) {
        SetPlane(0);
        BitmapPut(old_rect.xy.x0,old_rect.xy.y0,cursor_buf1,&fullscreen,A_REPLACE);
        SetPlane(1);
        BitmapPut(old_rect.xy.x0,old_rect.xy.y0,cursor_buf2,&fullscreen,A_REPLACE);

    }
}


/*===========================================================================*/
/* backups background of cursor location (given x/y is the logical position) */
/*===========================================================================*/
void BackupCursorPos(short x,short y) {
    //--------------------------------------------------------------
    // calculate physical x/y position and setup rectangle to backup
    //--------------------------------------------------------------
    x *= 8;
    x += tiles_offset_x+3;
    y *= 8;
    y += tiles_offset_y+3;
    old_rect.xy.x0 = x-1;
    old_rect.xy.y0 = y-1;
    old_rect.xy.x1 = x+9;
    old_rect.xy.y1 = y+13;

    //----------------------------
    // get contents of both planes
    //----------------------------
    SetPlane(0);
    BitmapGet(&old_rect,cursor_buf1);
    SetPlane(1);
    BitmapGet(&old_rect,cursor_buf2);
}


/*===========================================================================*/
/* output cursor sprite at logical position (x/y)                            */
/*===========================================================================*/
void OutputCursor(short x,short y) {
    //--------------------------------
    // calculate physical x/y position
    //--------------------------------
    x *= 8;
    x += tiles_offset_x+3;
    y *= 8;
    y += tiles_offset_y+3;

    Sprite8(x,y,12,mask_plane1,plane1,SPRT_AND);
    Sprite8(x,y,12,mask_plane2,plane2,SPRT_AND);
    Sprite8(x,y,12,pointer_plane1,plane1,SPRT_OR);
    Sprite8(x,y,12,pointer_plane2,plane2,SPRT_OR);
}

//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: cursor.c,v $
// Revision 1.1  2002/09/27 10:10:29  tnussb
// initial check-in to CVS repository
//
//
