/******************************************************************************
*
* project name: TI-MineSweeper
* filename:     timiner.c
* initial date: 04/07/2000
* authors:      thomas.nussbaumer@gmx.net (coding)
*               marcos.lopez@gmx.net (game play/graphics/BETA testing)
* description:  main file (contents most of the stuff)
*
* $Id: timiner.c,v 1.1 2002/09/27 10:10:30 tnussb Exp $
*
*******************************************************************************/

//------------------------------------------------------------------------------
//KEYS:
//
// [F1] or [HOME]          ... open a tile
// [F2], [MODE] or [ENTER] ... mark/unmark tile as bomb ([MODE] is for TI-89 only)
// [ESC]                   ... abort game/back to menu
//
//-----------------------------------------------------------------------------

#define SAVE_SCREEN
#define OPTIMIZE_ROM_CALLS
#define NO_EXIT_SUPPORT

#define USE_TI89
#define USE_TI92P
#define USE_V200

//#define USE_INITS_PATCH  // necessary for isprint() - don't know why ...

#include <tigcclib.h>

#define PROGRAM_NAME     "TI-MineSweeper"
#define PROGRAM_VERSION  "1.20"


short offsetx = 0;
short offsety = 0;


short GetUserInput(void);


#include "counter.c"     // data and output functions for counters
#include "tiles.c"       // data and output functions of tiles
#include "cursor.c"      // data and output functions for cursor
#include "smiley.c"      // data and output functions for smilies
#include "score.c"       // highscore handling functions
#include "logo.c"        // logo data and output functions

/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/* internal used structures and related defines                              */
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

//------------------------------
// description of one board tile
//------------------------------
typedef struct {
    char count;  // count of surrounding bombs (-1 == this tile is a bomb)
    char state;  // indicates state of the tile (see STATE_XXXX defines)
    char flag;   // flag used to mark tiles which graphics must be updated
    char unused;
} tile_t;

#define BOMB_VALUE       -1   // special count value which indicates a bomb

#define STATE_CLOSED      0   // tile is closed
#define STATE_OPENED      1   // tile is opened and it is a normal tile
#define STATE_MARKED      2   // tile is marked by user
#define STATE_INTERROGATE 3   // tile is "de-marked" by user
#define STATE_WRONGMARKED 4   // tile is wrongly marked
#define STATE_CLICKEDMINE 5   // tile is the bomb which was opened by the user

#define FLAG_NONE         0   // tile graphics is up-to-date
#define FLAG_TAGGED       1   // tile graphics must be updated


//---------------------------------------------
// describes one entry of the stack which is
// used by the "open board recursively" feature
//---------------------------------------------
typedef struct {
    short x;
    short y;
} stack_t;


/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/* some important defines                                                    */
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

//-------------------
// special key values
//-------------------
#define KEY_HOME         277 // doesn't exists on TI92 !!

//-------------------------
// some constant dimensions
//-------------------------
#define COUNTER_WIDTH    19
#define COUNTER_HEIGHT   15

#define SMILEY_WIDTH     17
#define SMILEY_HEIGHT    17

//----------------------
// maximum size of board
//----------------------
#define MAX_COLUMNS      18
#define MAX_LINES         8


/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/* global variables                                                          */
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/

SCR_RECT       fullscreen     = {{0,0,239,127}}; // used for fullscreen clipping
unsigned char* plane1         =  0; // start address of grayscale bitplane 1
unsigned char* plane2         =  0; // start address of grayscale bitplane 2
tile_t*        board          =  0; // the board itself (allocated dynamically)
stack_t*       stack          =  0; // stack used for open recursively feature
unsigned char* cursor_buf1    =  0; // background buffer for plane1 of cursor
unsigned char* cursor_buf2    =  0; // background buffer for plane2 of cursor
short          stack_counter  = -1; // counter of entries on the stack
short          nr_lines       =  0; // active board height
short          nr_columns     =  0; // active board width
short          nr_bombs       =  0; // active number of distributed bombs
short          cursor_x       =  0; // x coordinate of cursor
short          cursor_y       =  0; // y coordinate of cursor
unsigned short board_offset_x =  0; // x offset of board's upper left corner
unsigned short board_offset_y =  0; // y offset of board's upper left corner
unsigned short smiley_x       =  0; // x coordinate of smiley button
unsigned short smiley_y       =  0; // y coordinate of smiley button
unsigned short tiles_offset_x =  0; // x offset of upper left tile
unsigned short tiles_offset_y =  0; // y offset of upper left tile
short          draw_all       =  1; // indicator if all tiles should be drawn
counter_t      bomb_counter   = {0,0,0,{0,0,0}}; // counter for bombs
counter_t      time_counter   = {0,0,0,{0,0,0}}; // counter for seconds passed
short          mode           = 0;
short          timer_started  = 0;


#define BOARD(x,y) (board[(y)*MAX_COLUMNS+(x)])

/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/* some defines used in DrawColorLine() and FillColorRect() functions        */
/*---------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
#define COLOR_WHITE      0
#define COLOR_LIGHTGRAY  1
#define COLOR_DARKGRAY   2
#define COLOR_BLACK      3

#define LIGHT_PLANE      0
#define DARK_PLANE       1


/*===========================================================================*/
/* utility funcction to draw a string to both planes                         */
/*===========================================================================*/
void DrawStr2Planes(const char* s,short x,short y,short attr) {
    short i;
    for (i=0;i<2;i++) {
        SetPlane(i);
        DrawStr(x,y,s,attr);
    }
}


/*===========================================================================*/
/* utility function to draw a string centered in both planes                 */
/*===========================================================================*/
void DrawCenterString(const char* s,short y,short f,short attr) {
    FontSetSys(f);
    DrawStr2Planes(s,(LCD_WIDTH-DrawStrWidth(s,f))/2,y,attr);
}


/*===========================================================================*/
/* draw a line with given color from (x0,y0) to (x1,y1)                      */
/*===========================================================================*/
void DrawColorLine(short x0,short y0,short x1, short y1, short color) {
    SetPlane(LIGHT_PLANE);
    if (color == COLOR_WHITE || color == COLOR_DARKGRAY) {
        DrawLine(x0, y0, x1, y1, A_REVERSE);
    }
    else {
        DrawLine(x0, y0, x1, y1, A_NORMAL);
    }
    SetPlane(DARK_PLANE);
    if (color == COLOR_WHITE || color == COLOR_LIGHTGRAY) {
        DrawLine(x0, y0, x1, y1, A_REVERSE);
    }
    else {
        DrawLine(x0, y0, x1, y1, A_NORMAL);
    }
}


/*===========================================================================*/
/* fill a rectangle with given color                                         */
/*===========================================================================*/
void FillColorRect(short x1,short y1,short x2, short y2, short color) {
    SCR_RECT rect;

    rect.xy.x0 = x1;
    rect.xy.y0 = y1;
    rect.xy.x1 = x2;
    rect.xy.y1 = y2;

    SetPlane(LIGHT_PLANE);
    if (color == COLOR_WHITE || color == COLOR_DARKGRAY) {
        ScrRectFill(&rect,&fullscreen, A_REVERSE);
    }
    else {
        ScrRectFill(&rect,&fullscreen, A_NORMAL);
    }
    SetPlane(DARK_PLANE);
    if (color == COLOR_WHITE || color == COLOR_LIGHTGRAY) {
        ScrRectFill(&rect,&fullscreen, A_REVERSE);
    }
    else {
        ScrRectFill(&rect,&fullscreen, A_NORMAL);
    }
}


/*===========================================================================*/
/* initializes counter border                                                */
/* (simple helper function to save space)                                    */
/*===========================================================================*/
void InitCounterBorder(counter_t* p) {
    DrawColorLine(p->xpos-1, p->ypos-1,
                  p->xpos+COUNTER_WIDTH-1,p->ypos-1,
                  COLOR_DARKGRAY);
    DrawColorLine(p->xpos-1,p->ypos-1,
                  p->xpos-1,p->ypos+14,
                  COLOR_DARKGRAY);
    DrawColorLine(p->xpos+COUNTER_WIDTH,p->ypos,
                  p->xpos+COUNTER_WIDTH,p->ypos+14,
                  COLOR_WHITE);
    DrawColorLine(p->xpos, p->ypos+15,
                  p->xpos+COUNTER_WIDTH,p->ypos+15,
                  COLOR_WHITE);
}


/*===========================================================================*/
/* initializes the board graphics                                            */
/*===========================================================================*/
void InitScreen() {
    short board_height  = nr_lines   * 8 + 34;
    short board_width   = nr_columns * 8 + 12;

    //----------------------------
    // initialize global variables
    //----------------------------
    board_offset_x    = (LCD_WIDTH - board_width) >> 1;
    board_offset_y    = (LCD_HEIGHT - board_height) >> 1;
    bomb_counter.xpos = board_offset_x + 8;
    bomb_counter.ypos = board_offset_y + 7;
    time_counter.xpos = board_offset_x + board_width - COUNTER_WIDTH - 8;
    time_counter.ypos = board_offset_y + 7;
    smiley_x          = board_offset_x + ((board_width-SMILEY_WIDTH)>>1);
    smiley_y          = board_offset_y + 6;

    tiles_offset_x    = board_offset_x + 6;
    tiles_offset_y    = board_offset_y + 29;

    //----------------------------
    // clear both planes and set
    // draw_all flag to indicate
    // that each tile should be
    // drawn
    //----------------------------
    memset(GetPlane(0),0,LCD_SIZE);
    memset(GetPlane(1),0,LCD_SIZE);
    draw_all = 1;

    //----------------------------
    // draw background of board
    // and its borders
    // (DIRTY, but nevertheless
    // more efficiently than
    // using bitmaps)
    //----------------------------
    // general background
    FillColorRect(board_offset_x,
                  board_offset_y,
                  board_offset_x+board_width-1,
                  board_offset_y+board_height,
                  COLOR_LIGHTGRAY);

    // top lines
    DrawColorLine(board_offset_x,board_offset_y,
                  board_offset_x+board_width-2,board_offset_y,
                  COLOR_WHITE);

    DrawColorLine(board_offset_x,board_offset_y+1,
                  board_offset_x+board_width-3,
                  board_offset_y+1,
                  COLOR_WHITE);

    DrawColorLine(board_offset_x+5,board_offset_y+4,
                  board_offset_x+board_width-6,board_offset_y+4,
                  COLOR_DARKGRAY);

    // counter borders
    InitCounterBorder(&time_counter);
    InitCounterBorder(&bomb_counter);

    // left & right upper border
    DrawColorLine(board_offset_x+4,board_offset_y+4,
                  board_offset_x+4,board_offset_y+23,
                  COLOR_DARKGRAY);
    DrawColorLine(board_offset_x+board_width-5,board_offset_y+5,
                  board_offset_x+board_width-5,board_offset_y+23,
                  COLOR_WHITE);
    // horizontal lines of header
    DrawColorLine(board_offset_x+5,board_offset_y+24,
                  board_offset_x+board_width-5,board_offset_y+24,
                  COLOR_WHITE);
    DrawColorLine(board_offset_x+4,board_offset_y+27,
                  board_offset_x+board_width-6,board_offset_y+27,
                  COLOR_DARKGRAY);
    DrawColorLine(board_offset_x+4,board_offset_y+28,
                  board_offset_x+board_width-7,board_offset_y+28,
                  COLOR_DARKGRAY);
    // left border lines
    DrawColorLine(board_offset_x,board_offset_y,
                  board_offset_x,board_offset_y+board_height-1,
                  COLOR_WHITE);
    DrawColorLine(board_offset_x+1,board_offset_y,
                  board_offset_x+1,board_offset_y+board_height-2,
                  COLOR_WHITE);
    // bottom border lines
    DrawColorLine(board_offset_x+2,            board_offset_y+board_height-1,
                  board_offset_x+board_width-1,board_offset_y+board_height-1,
                  COLOR_DARKGRAY);
    DrawColorLine(board_offset_x+1,            board_offset_y+board_height,
                  board_offset_x+board_width-1,board_offset_y+board_height,
                  COLOR_DARKGRAY);
    // right border lines
    DrawColorLine(board_offset_x+board_width-2,board_offset_y+2,
                  board_offset_x+board_width-2,board_offset_y+board_height,
                  COLOR_DARKGRAY);
    DrawColorLine(board_offset_x+board_width-1,board_offset_y+1,
                  board_offset_x+board_width-1,board_offset_y+board_height,
                  COLOR_DARKGRAY);
    // center border left
    DrawColorLine(board_offset_x+4,board_offset_y+28,
                  board_offset_x+4,board_offset_y+board_height-5,
                  COLOR_DARKGRAY);
    DrawColorLine(board_offset_x+5,board_offset_y+28,
                  board_offset_x+5,board_offset_y+board_height-6,
                  COLOR_DARKGRAY);
    // center border bottom
    DrawColorLine(board_offset_x+6,board_offset_y+board_height-5,
                  board_offset_x+board_width-5,board_offset_y+board_height-5,
                  COLOR_WHITE);
    DrawColorLine(board_offset_x+5,board_offset_y+board_height-4,
                  board_offset_x+board_width-5,board_offset_y+board_height-4,
                  COLOR_WHITE);
    // center border right
    DrawColorLine(board_offset_x+board_width-6,board_offset_y+29,
                  board_offset_x+board_width-6,board_offset_y+board_height-5,
                  COLOR_WHITE);
    DrawColorLine(board_offset_x+board_width-5,board_offset_y+28,
                  board_offset_x+board_width-5,board_offset_y+board_height-5,
                  COLOR_WHITE);

    //----------------------------
    // setup rest of counters and
    // output them
    //----------------------------
    bomb_counter.value = nr_bombs;
    time_counter.value = 0;
    InitCounter(&bomb_counter);
    InitCounter(&time_counter);

    //----------------------------
    // output the happy smiley ...
    //----------------------------
    OutputSmiley(SMILEY_HAPPY);
}


/*===========================================================================*/
/* push entry on stack (helper function for OpenBoardRecursive())            */
/*===========================================================================*/
void PushStack(char x,char y) {
    stack_counter++;
#if 0
    assert(stack_counter < MAX_COLUMNS*MAX_LINES);
    assert(x < MAX_COLUMNS);
    assert(y < MAX_LINES);
#endif
    stack[stack_counter].x = x;
    stack[stack_counter].y = y;
}


/*===========================================================================*/
/* pop entry from stack (helper function for OpenBoardRecursive())           */
/*===========================================================================*/
stack_t* PopStack() {
    if (stack_counter < 0) return NULL;
    return &stack[stack_counter--];
}

/*===========================================================================*/
/* try to open board recursively (all tiles which should be opened gets      */
/* marked with FLAG_TAGGED)                                                  */
/*===========================================================================*/
void OpenBoardRecursive(char xstart,char ystart) {
    stack_t* p;

    stack_counter = -1; // initialize stack_counter

    PushStack(xstart,ystart);
    BOARD(xstart,ystart).flag = FLAG_TAGGED;

    while ((p = PopStack())) {
        char x = p->x;
        char y = p->y;

        if (BOARD(x,y).count > 0) continue;

        //-------------------------
        // check "line above" tiles
        //-------------------------
        if (y>0) {
            if (x>0) {
                if (BOARD(x-1,y-1).count >= 0           &&
                    BOARD(x-1,y-1).flag  != FLAG_TAGGED &&
                    BOARD(x-1,y-1).state == STATE_CLOSED)
                {
                    PushStack(x-1,y-1);
                    BOARD(x-1,y-1).flag  = FLAG_TAGGED;
                }
            }
            if (BOARD(x,y-1).count >= 0           &&
                BOARD(x,y-1).flag  != FLAG_TAGGED &&
                BOARD(x,y-1).state == STATE_CLOSED)
            {
                PushStack(x,y-1);
                BOARD(x,y-1).flag  = FLAG_TAGGED;
            }
            if (x<nr_columns-1) {
                if (BOARD(x+1,y-1).count >= 0           &&
                    BOARD(x+1,y-1).flag  != FLAG_TAGGED &&
                    BOARD(x+1,y-1).state == STATE_CLOSED)
                {
                    PushStack(x+1,y-1);
                    BOARD(x+1,y-1).flag  = FLAG_TAGGED;
                }
            }
        }

        //-------------------------------
        // check "within same line" tiles
        //-------------------------------
        if (x>0) {
            if (BOARD(x-1,y).count >= 0           &&
                BOARD(x-1,y).flag  != FLAG_TAGGED &&
                BOARD(x-1,y).state == STATE_CLOSED)
            {
                PushStack(x-1,y);
                BOARD(x-1,y).flag  = FLAG_TAGGED;
            }
        }

        if (x<nr_columns-1) {
            if (BOARD(x+1,y).count >= 0           &&
                BOARD(x+1,y).flag  != FLAG_TAGGED &&
                BOARD(x+1,y).state == STATE_CLOSED)
            {
                PushStack(x+1,y);
                BOARD(x+1,y).flag  = FLAG_TAGGED;
            }
        }

        //-------------------------------
        // check "line below" tiles
        //-------------------------------
        if (y<nr_lines-1) {
            if (x>0) {
                if (BOARD(x-1,y+1).count >= 0           &&
                    BOARD(x-1,y+1).flag  != FLAG_TAGGED &&
                    BOARD(x-1,y+1).state == STATE_CLOSED)
                {
                    PushStack(x-1,y+1);
                    BOARD(x-1,y+1).flag  = FLAG_TAGGED;
                }
            }
            if (BOARD(x,y+1).count >= 0           &&
                BOARD(x,y+1).flag  != FLAG_TAGGED &&
                BOARD(x,y+1).state == STATE_CLOSED)
            {
                PushStack(x,y+1);
                BOARD(x,y+1).flag  = FLAG_TAGGED;
            }
            if (x<nr_columns-1) {
                if (BOARD(x+1,y+1).count >= 0           &&
                    BOARD(x+1,y+1).flag  != FLAG_TAGGED &&
                    BOARD(x+1,y+1).state == STATE_CLOSED)
                {
                    PushStack(x+1,y+1);
                    BOARD(x+1,y+1).flag  = FLAG_TAGGED;
                }
            }
        }
    }

}


/*===========================================================================*/
/* allocates memory for board, stack and cursor background storage           */
/* NOTE: allocates as much memory as necessary for largest board, so each    */
/*       one will fit without changing the allocating procedure.             */
/*===========================================================================*/
short AllocateMemory(void) {
    //------------------------------------------------------------
    // allocate memory for board
    //------------------------------------------------------------
    if (!(board = (tile_t*)malloc(sizeof(tile_t)*MAX_COLUMNS*MAX_LINES))) return 0;

    //-----------------------------------------------------------
    // allocate stack used for "turn more than one piece" feature
    //-----------------------------------------------------------
    if (!(stack = (stack_t*)malloc(sizeof(stack_t)*MAX_COLUMNS*MAX_LINES))) return 0;

    //-----------------------------------------------------------
    // allocate space for cursor background buffer
    // NOTE: we don't need really 250 bytes for each, but I'm too
    //       lazy for now to calculate its correct value
    //-----------------------------------------------------------
    if (!(cursor_buf1 = (unsigned char*)malloc(250))) return 0;
    if (!(cursor_buf2 = (unsigned char*)malloc(250))) return 0;

    return 1;
}


/*===========================================================================*/
/* free's allocated memory again                                             */
/*===========================================================================*/
void FreeMemory(void) {
    if (board)       free(board);
    if (stack)       free(stack);
    if (cursor_buf1) free(cursor_buf1);
    if (cursor_buf2) free(cursor_buf2);
    stack       = NULL;
    board       = NULL;
    cursor_buf1 = NULL;
    cursor_buf2 = NULL;
}


/*===========================================================================*/
/* set all board entries to zero (initial state)                             */
/*===========================================================================*/
void ClearBoard(void) {
    memset(board,0,sizeof(tile_t)*MAX_COLUMNS*MAX_LINES);
}


/*===========================================================================*/
/* distributes bombs in level (loop until all bombs are distributed)         */
/*===========================================================================*/
void DistributeBombs() {
    short tmp = nr_bombs;
    do {
        short x = random(nr_columns);
        short y = random(nr_lines);
        if (BOARD(x,y).count == 0) {
            BOARD(x,y).count = BOMB_VALUE;
            tmp--;
        }
    }
    while (tmp > 0);
}


/*===========================================================================*/
/* calculate counts of tiles                                                 */
/*===========================================================================*/
void CalculateCounts() {
    short x;
    short y;
    short act;

    for (y=0;y<nr_lines;y++) {
        for (x=0;x<nr_columns;x++) {
            if (BOARD(x,y).count == BOMB_VALUE) continue;

            act = 0;
            //-------------------------
            // check "line above" tiles
            //-------------------------
            if (y>0) {
                if (x>0 && BOARD(x-1,y-1).count == BOMB_VALUE)            act++;
                if (BOARD(x,y-1).count == BOMB_VALUE)                     act++;
                if (x<nr_columns-1 && BOARD(x+1,y-1).count == BOMB_VALUE) act++;
            }

            //-------------------------------
            // check "within same line" tiles
            //-------------------------------
            if (x>0 && BOARD(x-1,y).count == BOMB_VALUE) act++;
            if (x<nr_columns-1 && BOARD(x+1,y).count == BOMB_VALUE) act++;

            //-------------------------------
            // check "line below" tiles
            //-------------------------------
            if (y<nr_lines-1) {
                if (x>0 && BOARD(x-1,y+1).count == BOMB_VALUE)            act++;
                if (BOARD(x,y+1).count == BOMB_VALUE)                     act++;
                if (x<nr_columns-1 && BOARD(x+1,y+1).count == BOMB_VALUE) act++;
            }

            BOARD(x,y).count = act;
        }
    }
}


/*===========================================================================*/
/* check for finish - returns 1 if finished                                  */
/* NOTE: we could optimize this by using counters which gots decremented     */
/*       during a change, so that we haven't to scan the whole board again.  */
/*       But our calculators are that fast that we don't need such           */
/*       optimizations for a simple game like TI-Miner !!                    */
/*===========================================================================*/
short IsFinished() {
    short x;
    short y;
    short opened = 0;

    for (y=0;y<nr_lines;y++) {
        for (x=0;x<nr_columns;x++) {
            if (BOARD(x,y).count >= 0 && BOARD(x,y).state == STATE_OPENED) opened++;
        }
    }

    if (opened == nr_lines*nr_columns - nr_bombs) return 1;

    return 0;
}


/*===========================================================================*/
/* if tiles are tagged for update and they are closed or "un-marked" by the  */
/* user, this function will set them to state STATE_OPENED                   */
/*===========================================================================*/
void OpenTaggedTiles() {
    short x;
    short y;

    for (y=0;y<nr_lines;y++) {
        for (x=0;x<nr_columns;x++) {
            if (BOARD(x,y).flag == FLAG_TAGGED) {
                if (BOARD(x,y).state == STATE_CLOSED ||
                    BOARD(x,y).state == STATE_INTERROGATE)
                {
                    BOARD(x,y).state = STATE_OPENED;
                }
            }
        }
    }
}


/*===========================================================================*/
/* open all bomb fields and mark them for display                            */
/*===========================================================================*/
void ProcessAllBombs() {
    short x;
    short y;

    for (y=0;y<nr_lines;y++) {
        for (x=0;x<nr_columns;x++) {
            if (x==cursor_x && y==cursor_y) {
                BOARD(x,y).state = STATE_CLICKEDMINE;
                BOARD(x,y).flag  = FLAG_TAGGED;
            }
            else if (BOARD(x,y).count == -1) {
                switch (BOARD(x,y).state) {
                    case STATE_CLOSED:
                    case STATE_INTERROGATE:
                        BOARD(x,y).state = STATE_OPENED;
                        BOARD(x,y).flag  = FLAG_TAGGED;
                        break;
                    case STATE_MARKED:
                        BOARD(x,y).state = STATE_OPENED;
                        BOARD(x,y).flag  = FLAG_TAGGED;
                        break;
                }
            }
            else {
                if (BOARD(x,y).state == STATE_MARKED) {
                    BOARD(x,y).state = STATE_WRONGMARKED;
                    BOARD(x,y).flag  = FLAG_TAGGED;
                }
            }

        }
    }
}


/*===========================================================================*/
/* process board drawing                                                     */
/*===========================================================================*/
void DrawBoard(void) {
    short x;
    short y;

    RestoreOldCursorPos();

    for (y=0;y<nr_lines;y++) {
        for (x=0;x<nr_columns;x++) {
            if (BOARD(x,y).flag == FLAG_TAGGED || draw_all) {
                BOARD(x,y).flag = FLAG_NONE;
                switch(BOARD(x,y).state) {
                    case STATE_CLOSED:
                        OutputMiscTile(TYPE_UNDISCOVERED,x,y);
                        break;
                    case STATE_INTERROGATE:
                        OutputMiscTile(TYPE_INTERROGATION,x,y);
                        break;
                    case STATE_OPENED:
                        if (BOARD(x,y).count >= 0) OutputCountTile(BOARD(x,y).count,x,y);
                        else                       OutputMiscTile(TYPE_MINE,x,y);
                        break;
                    case STATE_MARKED:
                        OutputMiscTile(TYPE_MARKED,x,y);
                        break;
                    case STATE_WRONGMARKED:
                        OutputMiscTile(TYPE_WRONGMARK,x,y);
                        break;
                    case STATE_CLICKEDMINE:
                        OutputMiscTile(TYPE_CLICKEDMINE,x,y);
                        break;
                }
            }
        }
    }

    draw_all = 0;
    BackupCursorPos(cursor_x,cursor_y);
    OutputCursor(cursor_x,cursor_y);
}


/*===========================================================================*/
/* get input key pressed by user                                             */
/*                                                                           */
/* replacement for ngetchx() and/or GKeyIn(), because:                       */
/*                                                                           */
/* (1) ngetchx() won't check the APD timer value                             */
/* (2) GKeyIn() destroys the display due to setting the status display       */
/*===========================================================================*/
short GetUserInput(void) {
    void *kbq = kbd_queue();
    short key;

    //-------------------------------------------
    // restart the APD timer now
    //-------------------------------------------
    OSTimerRestart(APD_TIMER);

    while (1) {
        //---------------------------------------
        // wait until we get a key
        // if the APD timer expires -> turn
        // calculator off
        //---------------------------------------
        while (OSdequeue(&key, kbq)) {
            if (OSTimerExpired(APD_TIMER)) off();

            //-------------------------
            // handling of time counter
            //-------------------------
            if (timer_started) { // only if we have it stared by our own
                if (OSTimerExpired(USER_TIMER)) {
                    if (time_counter.value<999) {
                        time_counter.value++;
                        UpdateCounter(&time_counter);
                    }
                }
            }
        }

        //---------------------------------------
        // handle calculator-off key combinations
        // if the APD timer expires -> turn
        // calculator off
        //---------------------------------------
        if (key == 4363 || (TI89 && key == 16651) || (!TI89 && key == 8459)) {
            off();
        }
        else {
            return key & 0xf7ff;  // mask out 0x800 (repeat flag)
        }
    }
}


/*===========================================================================*/
/* handles opening of a tile (returns 1 if a bomb tile is opened)            */
/*===========================================================================*/
short HandleOpenTile() {
    switch(BOARD(cursor_x,cursor_y).count) {
        case 0:
            OpenBoardRecursive(cursor_x,cursor_y);
            OpenTaggedTiles();
            return 0;
        case -1:
            BOARD(cursor_x,cursor_y).flag  = FLAG_TAGGED;
            BOARD(cursor_x,cursor_y).state = STATE_CLICKEDMINE;
            return 1;
        default:
            BOARD(cursor_x,cursor_y).flag  = FLAG_TAGGED;
            BOARD(cursor_x,cursor_y).state = STATE_OPENED;
            return 0;
    }
}


/*===========================================================================*/
/* input dialog for highscore player's name                                  */
/*===========================================================================*/
void GetPlayerName() {
    char s[PLAYER_NAME_LENGTH] = "                   ";
    short  position = 0;
    short  input;

    FillColorRect(8+offsetx,40+offsety,151+offsetx,60+offsety,COLOR_BLACK);
    FillColorRect(9+offsetx,41+offsety,150+offsetx,59+offsety,COLOR_WHITE);

    DrawCenterString("New Highscore! Please enter your name:",43+offsety,F_4x6,A_REPLACE);

    do {
        input = GetUserInput();
        if (input == KEY_BACKSPACE) {
           if (position > 0) {
              s[--position] = 0;
              FillColorRect(10+offsetx,52+offsety,149+offsetx,58+offsety,COLOR_WHITE);
              DrawCenterString(s,52+offsety,F_4x6,A_REPLACE);
           }
        }
        else if (isprint(input)) {
           if (position < PLAYER_NAME_LENGTH-2) {
              s[position++] = input;
              s[position]   = 0;
              FillColorRect(10+offsetx,52+offsety,149+offsetx,58+offsety,COLOR_WHITE);
              DrawCenterString(s,52+offsety,F_4x6,A_REPLACE);
           }
        }
    }
    while (input != KEY_ENTER && input != KEY_ESC);

    memcpy(scores[mode].name,s,PLAYER_NAME_LENGTH);
}


/*===========================================================================*/
/* major game loop                                                           */
/*===========================================================================*/
short PlayTheGame(short lines,short columns,short bombs) {
    short bomb_opened = 0;
    short input;

    //----------------------
    // setup game parameters
    //----------------------
    timer_started = 0;
    nr_lines      = lines;
    nr_columns    = columns;
    nr_bombs      = bombs;
    cursor_x      = 0;
    cursor_y      = 0;
    InitCursorBuffer();

    //-----------------------------------------------------------------------
    // initialize the board (clear it/distribute the bombs/calculate counts)
    //-----------------------------------------------------------------------
    ClearBoard();
    DistributeBombs(nr_bombs);
    CalculateCounts();

    //-----------------------------------------------------
    // setup graphics related globals and initialize screen
    //-----------------------------------------------------
    InitScreen();

    //---------------------------------------
    // now loop until the user gets tired ...
    //---------------------------------------
    do {
        DrawBoard();

        input = GetUserInput();

        if (input == KEY_UP)      cursor_y--;
        if (cursor_y<0)           cursor_y = nr_lines-1;
        if (input == KEY_DOWN)    cursor_y++;
        if (cursor_y>=nr_lines)   cursor_y = 0;
        if (input == KEY_LEFT)    cursor_x--;
        if (cursor_x<0)           cursor_x = nr_columns-1;
        if (input == KEY_RIGHT)   cursor_x++;
        if (cursor_x>=nr_columns) cursor_x = 0;

        //-----------------------------------
        // user wants to turn the active tile
        //-----------------------------------
        if (input == KEY_F1 || input == KEY_HOME) {
            //---------------------------------------------------
            // setup timer to be triggered every second (20*50ms)
            // after user's first "turn tile" key press
            //---------------------------------------------------
            if (!timer_started) {
                OSFreeTimer(USER_TIMER);
                OSRegisterTimer(USER_TIMER,20);
                timer_started = 1;
            }
            if (BOARD(cursor_x,cursor_y).state == STATE_INTERROGATE) {
                BOARD(cursor_x,cursor_y).state = STATE_CLOSED;
            }
            if (BOARD(cursor_x,cursor_y).state == STATE_CLOSED ||
                BOARD(cursor_x,cursor_y).state == STATE_INTERROGATE)
            {
                bomb_opened = HandleOpenTile();
            }
        }
        //-------------------------------------------
        // user wants to mark/un-mark the active tile
        //-------------------------------------------
        if (input == KEY_ENTER || input == KEY_F2 || input == KEY_MODE) {
            switch (BOARD(cursor_x,cursor_y).state) {
                case STATE_CLOSED:
                    if (bomb_counter.value != 0) {
                        BOARD(cursor_x,cursor_y).flag  = FLAG_TAGGED;
                        BOARD(cursor_x,cursor_y).state = STATE_MARKED;
                        bomb_counter.value--;
                        UpdateCounter(&bomb_counter);
                    }
                    break;
                case STATE_MARKED:
                    BOARD(cursor_x,cursor_y).flag  = FLAG_TAGGED;
                    BOARD(cursor_x,cursor_y).state = STATE_INTERROGATE;
                    bomb_counter.value++;
                    UpdateCounter(&bomb_counter);
                    break;
                case STATE_INTERROGATE:
                    BOARD(cursor_x,cursor_y).flag  = FLAG_TAGGED;
                    BOARD(cursor_x,cursor_y).state = STATE_CLOSED;
                    break;
            }
        }

    }
    while(!IsFinished() && !bomb_opened && input != KEY_ESC);

    //------------------------------------------------------------
    // stop timer
    //------------------------------------------------------------
    if (timer_started) {
        OSFreeTimer(USER_TIMER);
        timer_started = 0;
    }

    //------------------------------------------------------------
    // if user lost the game -> show sad smiley and open all bombs
    //------------------------------------------------------------
    if (bomb_opened) {
        OutputSmiley(SMILEY_SAD);
        ProcessAllBombs();
    }
    //------------------------------------------------------------
    // if user aborts -> show the surprised smiley
    //------------------------------------------------------------
    else if (input == KEY_ESC) {
        OutputSmiley(SMILEY_SURPRISED);
    }
    else {
        OutputSmiley(SMILEY_WIN);
        //----------------------------------------------------
        // handle new highscore !!
        //----------------------------------------------------
        if (scores[mode].time >= time_counter.value) {
            DrawBoard();
            GetPlayerName();
            scores[mode].time = time_counter.value;
            WriteHighScores();
            return KEY_ENTER;
        }
    }

    //--------------------------------------------------------
    // update board ones more, remove the timer and return the
    // next keypress of the user
    //--------------------------------------------------------
    DrawBoard();

    return GetUserInput();
}


/*===========================================================================*/
/* invert a menu entry                                                       */
/*===========================================================================*/
void InvertMenuEntry(short i) {
    SCR_RECT rect;
    short    loop;

    rect.xy.x0 = 40       + offsetx;
    rect.xy.y0 = 16 + 9*i + offsety;
    rect.xy.x1 = 119      + offsetx;
    rect.xy.y1 = 24 + 9*i + offsety;

    for (loop=0;loop<2;loop++) {
        SetPlane(loop);
        ScrRectFill(&rect,&fullscreen,A_XOR);
    }
}


/*===========================================================================*/
/* handles menu (returns 1 if game should be started)                        */
/*===========================================================================*/
short HandleMenu() {
    char  stmp[100];
    short input;
    short selection = mode;

    memset(GetPlane(0),0,LCD_SIZE);
    memset(GetPlane(1),0,LCD_SIZE);

    FillColorRect(offsetx,offsety,159+offsetx,14+offsety,COLOR_BLACK);
    DrawCenterString(PROGRAM_NAME" v"PROGRAM_VERSION,1+offsety,F_6x8,A_XOR);
    DrawCenterString("(c) TI-Chess Team 2002",9+offsety,F_4x6,A_XOR);

    FillColorRect(offsetx,62+offsety,159+offsetx,71+offsety,COLOR_BLACK);
    DrawCenterString("Highscores",63+offsety,F_6x8,A_XOR);

    //--------------------------------------------------------------
    // dirty hack to get the "Play 8x8 string aligned with the other
    // ones !!!
    //--------------------------------------------------------------
    FontSetSys(F_6x8);
    DrawStr2Planes("Play 8x8",
                   (LCD_WIDTH-DrawStrWidth("Play 8x08",F_6x8))/2,
                   17+offsety,A_REPLACE);

    DrawCenterString("Play 8x12",26+offsety,F_6x8,A_REPLACE);
    DrawCenterString("Play 8x16",35+offsety,F_6x8,A_REPLACE);
    DrawCenterString("Play 8x18",44+offsety,F_6x8,A_REPLACE);
    DrawCenterString("Exit",     53+offsety,F_6x8,A_REPLACE);

    FontSetSys(F_4x6);
    sprintf(stmp,"Mode:8x8      Time:%03d    %s",scores[0].time,scores[0].name);
    DrawStr2Planes(stmp, 1+offsetx,73+offsety,A_REPLACE);
    sprintf(stmp,"Mode:8x12    Time:%03d    %s",scores[1].time,scores[1].name);
    DrawStr2Planes(stmp, 1+offsetx,80+offsety,A_REPLACE);
    sprintf(stmp,"Mode:8x16    Time:%03d    %s",scores[2].time,scores[2].name);
    DrawStr2Planes(stmp, 1+offsetx,87+offsety,A_REPLACE);
    sprintf(stmp,"Mode:8x18    Time:%03d    %s",scores[3].time,scores[3].name);
    DrawStr2Planes(stmp, 1+offsetx,94+offsety,A_REPLACE);

    InvertMenuEntry(selection);
    do {
        input = GetUserInput();
        if (input == KEY_DOWN) {
            InvertMenuEntry(selection);
            selection++;
            if (selection > 4) selection = 0;
            InvertMenuEntry(selection);
        }
        if (input == KEY_UP) {
            InvertMenuEntry(selection);
            selection--;
            if (selection < 0) selection = 4;
            InvertMenuEntry(selection);
        }
    }
    while (input != KEY_ENTER && input != KEY_ESC);

    if (input == KEY_ENTER) {
        if (selection == 4) return 0;
        mode = selection;
        return 1;
    }

    return 0;
}


/*===========================================================================*/
/* main routine: where all the fun starts ...                                */
/*===========================================================================*/
void _main(void) {
    //------------------------------------------
    // turn on grayscale graphics subsystem
    //------------------------------------------
    if (!GrayOn()) {
        return;
    }

    timer_started = 0;

    offsetx = (TI89) ? 0:40;
    offsety = (TI89) ? 0:14;

    //------------------------------------------
    // try to load highscores from variable or
    // initialize with defaults
    //------------------------------------------
    LoadHighScores();

    //------------------------------------------
    // allocates memory
    //------------------------------------------
    if (!AllocateMemory()) {
        FreeMemory();
        GrayOff();
        ClearScreen();
        DrawStr(0,0,"Error: OutOfMemory",A_REPLACE);
        ngetchx();
        return;
    }

    //------------------------------------------
    // setup graphics plane pointers
    //------------------------------------------
    plane1 = GetPlane(0);
    plane2 = GetPlane(1);

    //------------------------------------------
    // output logo
    //------------------------------------------
    Logo();

    //------------------------------------------
    // initialize random number generator
    //------------------------------------------
    randomize();

    //-------------------------------------------
    // show menu and start game depending on mode
    //-------------------------------------------
    while (HandleMenu()) {
        switch(mode) {
            case 0:  while (PlayTheGame(8, 8,10) != KEY_ESC);  // Mode  9x8 - 10 bombs
                     break;
            case 1:  while (PlayTheGame(8,12,15) != KEY_ESC);  // Mode 12x8 - 15 bombs
                     break;
            case 2:  while (PlayTheGame(8,16,20) != KEY_ESC);  // Mode 16x8 - 20 bombs
                     break;
            default: while (PlayTheGame(8,18,25) != KEY_ESC);  // Mode 18x8 - 25 bombs
                     break;
        }
    }

    //---------------------------------------------
    // free memory and say goodbye =8-)
    //---------------------------------------------
    FreeMemory();

    GrayOff();
}

//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: timiner.c,v $
// Revision 1.1  2002/09/27 10:10:30  tnussb
// initial check-in to CVS repository
//
//
